@@ -1,5 +1,6 @@
 # Copyright 2020 Michael Still
 
+from collections import defaultdict
 import json
 
 from shakenfist import etcd
@@ -16,20 +17,21 @@ def clean_events_mesh_operations(etcd_client):
     # can return at one time.
 
     # Save time and use the already available etcdctl client.
-    net_keys, stderr = util_process.execute(None,
-                                            'etcdctl get --prefix /sf/event/network/ | grep sf/event',
-                                            check_exit_code=[0, 1])
+    net_keys, stderr = util_process.execute(
+        None,
+        'etcdctl get --prefix /sf/event/network/ | grep sf/event',
+        check_exit_code=[0, 1])
     if stderr:
         print('ERROR: Unable to retrieve network keys:%s' % stderr)
         return
 
     # Split network events into networks
-    network_events = {}
+    network_events = defaultdict(list)
     for key in net_keys.split('\n'):
         if not key:
             continue
         _blank, _sf, _event, _network, uuid, _time = key.split('/')
-        network_events.setdefault(uuid, []).append(key)
+        network_events[uuid].append(key)
 
     # Delete all but last 50 events
     count = 0
@@ -44,24 +46,17 @@ def clean_events_mesh_operations(etcd_client):
 def main():
     etcd_client = etcd.WrappedEtcdClient()
 
-    releases = {}
-    old_style_nodes = []
-
+    releases = defaultdict(int)
     for data, _ in etcd_client.get_prefix('/sf/node/'):
         n = json.loads(data.decode('utf-8'))
 
         observed = etcd_client.get(
             '/sf/attribute/node/%s/observed' % n['fqdn'])
-        if observed:
-            # New style node
-            observed = json.loads(observed[0].decode('utf-8'))
-            release = observed['release']
-        else:
-            # Old style node
-            release = n.get('version', 'unknown')
-            old_style_nodes.append(n['fqdn'])
+        if not observed:
+            continue
 
-        releases.setdefault(release, 0)
+        observed = json.loads(observed[0].decode('utf-8'))
+        release = observed['release']
         releases[release] += 1
 
     print('Deployed releases:')
@@ -71,9 +66,9 @@ def main():
 
     min_release = None
     if not releases:
-        min_release = '0.2'
+        min_release = '0.4'
     elif 'unknown' in releases:
-        min_release = '0.2'
+        min_release = '0.4'
     else:
         min_release = sorted(releases)[0]
     print('Minimum release is %s' % min_release)
@@ -86,6 +81,35 @@ def main():
         if minor <= 4:
             clean_events_mesh_operations(etcd_client)
 
+            for data, metadata in etcd_client.get_prefix('/sf/instance/'):
+                i = json.loads(data.decode('utf-8'))
+                changed = False
+
+                # Find version 3 instances and migrate them to version 4
+                if i.get('version') == 3:
+                    i['configdrive'] = 'openstack-disk'
+                    i['version'] = 4
+                    changed = True
+
+                # Find version 4 instances and migrate them to version 5
+                if i.get('version') == 4:
+                    i['nvram_template'] = None
+                    i['secure_boot'] = False
+                    i['version'] = 5
+                    changed = True
+
+                # Find version 5 instances and migrate them to version 6
+                if i.get('version') == 5:
+                    i['machine_type'] = 'pc'
+                    i['version'] = 6
+                    changed = True
+
+                if changed:
+                    print('--> Upgraded instance %s to version %d'
+                          % (i['uuid'], i['version']))
+                    etcd_client.put(metadata['key'],
+                                    json.dumps(i, indent=4, sort_keys=True))
+
 
 if __name__ == '__main__':
     main()
