@@ -4,25 +4,30 @@ from functools import partial
 from uuid import uuid4
 
 from shakenfist import baseobject
+from shakenfist.baseobject import (
+    DatabaseBackedObject as dbo,
+    DatabaseBackedObjectIterator as dbo_iter)
 from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
+from shakenfist.ipmanager import IPManager
 from shakenfist import logutil
-from shakenfist import util
+from shakenfist.tasks import DefloatNetworkInterfaceTask
+from shakenfist.util import network as util_network
 
 
 LOG, _ = logutil.setup(__name__)
 
 
-class NetworkInterface(baseobject.DatabaseBackedObject):
+class NetworkInterface(dbo):
     object_type = 'networkinterface'
     current_version = 2
     state_targets = {
-        None: ('initial', ),
-        'initial': ('created', 'deleted', 'error'),
-        'created': ('created', 'deleted', 'error'),
-        'error': ('deleted', 'error'),
-        'deleted': (),
+        None: (dbo.STATE_INITIAL, ),
+        dbo.STATE_INITIAL: (dbo.STATE_CREATED, dbo.STATE_DELETED, dbo.STATE_ERROR),
+        dbo.STATE_CREATED: (dbo.STATE_DELETED, dbo.STATE_ERROR),
+        dbo.STATE_ERROR: (dbo.STATE_DELETED, dbo.STATE_ERROR),
+        dbo.STATE_DELETED: (),
     }
 
     def __init__(self, static_values):
@@ -39,10 +44,10 @@ class NetworkInterface(baseobject.DatabaseBackedObject):
     @classmethod
     def new(cls, interface_uuid, netdesc, instance_uuid, order):
         if 'macaddress' not in netdesc or not netdesc['macaddress']:
-            possible_mac = util.random_macaddr()
+            possible_mac = util_network.random_macaddr()
             mac_iface = {'interface_uuid': interface_uuid}
             while not etcd.create('macaddress', None, possible_mac, mac_iface):
-                possible_mac = util.random_macaddr()
+                possible_mac = util_network.random_macaddr()
             netdesc['macaddress'] = possible_mac
 
         if not interface_uuid:
@@ -63,16 +68,9 @@ class NetworkInterface(baseobject.DatabaseBackedObject):
             }
         )
 
-        LOG.with_fields({
-            'networkinterface': interface_uuid,
-            'instance_uuid': instance_uuid,
-            'network_uuid': netdesc['network_uuid']
-        }).debug('Network interface created')
-
         ni = NetworkInterface.from_db(interface_uuid)
         ni._db_set_attribute('floating', {'floating_address': None})
-        ni.state = 'initial'
-        ni.add_event('db record creation', None)
+        ni.state = NetworkInterface.STATE_INITIAL
 
         # TODO(andy): Integrate metadata into each object type
         # Initialise metadata
@@ -142,9 +140,23 @@ class NetworkInterface(baseobject.DatabaseBackedObject):
 
     @floating.setter
     def floating(self, address):
-        if self.floating.get('floating_ip') is not None:
+        if address and self.floating.get('floating_address') is not None:
             raise exceptions.NetworkInterfaceAlreadyFloating()
-        self._db_set_attribute('floating', {'floating_ip': address})
+        self._db_set_attribute('floating', {'floating_address': address})
+
+    def delete(self):
+        if self.floating['floating_address']:
+            etcd.enqueue(
+                'networknode',
+                DefloatNetworkInterfaceTask(self.network_uuid, self.uuid))
+
+        with db.get_lock('ipmanager', None, self.network_uuid,
+                         ttl=120, op='Release fixed IP'):
+            ipm = IPManager.from_db(self.network_uuid)
+            ipm.release(self.ipv4)
+            ipm.persist()
+
+        self.state = dbo.STATE_DELETED
 
     def hard_delete(self):
         etcd.delete('macaddress', None, self.macaddr)
@@ -154,14 +166,10 @@ class NetworkInterface(baseobject.DatabaseBackedObject):
         db.delete_metadata('networkinterface', self.uuid)
 
 
-class NetworkInterfaces(baseobject.DatabaseBackedObjectIterator):
+class NetworkInterfaces(dbo_iter):
     def __iter__(self):
         for _, ni in etcd.get_all('networkinterface', None):
-            ni = NetworkInterface.from_db(ni['uuid'])
-            if not ni:
-                continue
-
-            out = self.apply_filters(ni)
+            out = self.apply_filters(NetworkInterface(ni))
             if out:
                 yield out
 
@@ -183,10 +191,6 @@ def interfaces_for_instance(instance):
         nis[ni.order] = ni
         loggable_nis[ni.order] = str(ni)
 
-    LOG.with_fields({
-        'instance': instance,
-        'interfaces': loggable_nis
-    }).debug('Found %d interfaces' % len(nis))
     for order in sorted(nis.keys()):
         yield nis[order]
 
