@@ -1,9 +1,9 @@
 import mock
 
 from shakenfist.baseobject import State
-from shakenfist.config import SFConfigBase
+from shakenfist.config import BaseSettings
 from shakenfist.daemons import cleaner
-from shakenfist.tests import test_shakenfist
+from shakenfist.tests import base
 
 
 class FakeLibvirt(object):
@@ -58,7 +58,7 @@ def fake_exists(path):
     return True
 
 
-class FakeConfig(SFConfigBase):
+class FakeConfig(BaseSettings):
     NODE_NAME: str = 'abigcomputer'
     STORAGE_PATH: str = '/srv/shakenfist'
     LOGLEVEL_CLEANER: str = 'debug'
@@ -98,12 +98,12 @@ def fake_get(objecttype, subtype, name):
     return val
 
 
-class CleanerTestCase(test_shakenfist.ShakenFistTestCase):
+class CleanerTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(CleanerTestCase, self).setUp()
 
         self.libvirt = mock.patch(
-            'shakenfist.util.get_libvirt',
+            'shakenfist.util.libvirt.get_libvirt',
             return_value=FakeLibvirt())
         self.mock_libvirt = self.libvirt.start()
         self.addCleanup(self.libvirt.stop)
@@ -116,19 +116,25 @@ class CleanerTestCase(test_shakenfist.ShakenFistTestCase):
         self.mock_config = self.config.start()
         self.addCleanup(self.config.stop)
 
-    @mock.patch('shakenfist.virt.Instance.error',
+    @mock.patch('shakenfist.instance.Instance.state',
+                new_callable=mock.PropertyMock)
+    @mock.patch('shakenfist.baseobject.State.value',
+                new_callable=mock.PropertyMock)
+    @mock.patch('shakenfist.instance.Instance.error',
                 new_callable=mock.PropertyMock)
     @mock.patch('shakenfist.etcd.get', side_effect=fake_get)
     @mock.patch('shakenfist.db.get_lock')
     @mock.patch('shakenfist.node.Node.observe_this_node')
     @mock.patch('shakenfist.db.add_event')
-    @mock.patch('shakenfist.virt.Instance._db_get', side_effect=fake_instance_get)
+    @mock.patch('shakenfist.instance.Instance._db_get', side_effect=fake_instance_get)
     @mock.patch('shakenfist.etcd.put', side_effect=fake_put)
     @mock.patch('os.path.exists', side_effect=fake_exists)
     @mock.patch('time.time', return_value=7)
     def test_update_power_states(self, mock_time, mock_exists, mock_put,
                                  mock_get_instance, mock_event, mock_see,
-                                 mock_lock, mock_etcd_get, mock_error):
+                                 mock_lock, mock_etcd_get, mock_error,
+                                 mock_state_value, mock_state):
+        mock_state_value.return_value = 'created'
 
         m = cleaner.Monitor('cleaner')
         m._update_power_states()
@@ -151,8 +157,6 @@ class CleanerTestCase(test_shakenfist.ShakenFistTestCase):
                  'power_state', {'power_state': 'off'}),
                 ('attribute/instance', 'crashed',
                  'power_state', {'power_state': 'crashed'}),
-                ('attribute/instance', 'crashed',
-                 'state', State('error', 7)),
                 ('attribute/instance', 'paused',
                  'power_state', {'power_state': 'paused'}),
                 ('attribute/instance', 'suspended',
@@ -160,7 +164,5 @@ class CleanerTestCase(test_shakenfist.ShakenFistTestCase):
                 ('attribute/instance', 'foo',
                  'power_state', {'power_state': 'off'}),
                 ('attribute/instance', 'bar',
-                 'power_state', {'power_state': 'off'}),
-                ('attribute/instance', 'nofiles',
-                 'state', State('error', 7))
+                 'power_state', {'power_state': 'off'})
             ], result)
